# coding: UTF-8

require 'date'
require 'RmenuFilePathMixin'
require 'RmenuJsonMixin'

class B_DailyStockUpdate_model

  include RmenuFilePathMixin
  include RmenuJsonMixin

  def initialize(rmenu_db, sql_data, request_data)
    begin
      # initialize 開始ログを出力する
      $Mlog.debug("B_DailyStockUpdate_model") {"initialize start"}

      @rmenu_db       = rmenu_db
      @sql_data       = sql_data
      @request_data   = request_data

      # initialize 終了ログを出力する
      $Mlog.debug("B_DailyStockUpdate_model") {"initialize normal end"}
    rescue Exception
      # エラーログを出力する
      $Mlog.error("B_DailyStockUpdate_model") {"initialize exception: #{$!}"}
      raise
    end
  end
	

  # 処理実行可否判定
  def check実行可否判定(sqlIdName, sqlIO, sqlItemName)
    $Mlog.debug("B_DailyStockUpdate_model") {"check実行可否判定 start"}

    status = "OK"

    cond_chunk = getJsonChunkById(@sql_data, "sqls", sqlIdName, sqlIO, "record")
    checkValue = cond_chunk[sqlItemName]["value"][0]
		
    if checkValue == ""
      status = "PASS"
    end

    $Mlog.debug("B_DailyStockUpdate_model") {"check実行可否判定: #{sqlIdName} #{sqlIO} #{sqlItemName} #{checkValue} #{status}"}
    $Mlog.debug("B_DailyStockUpdate_model") {"check実行可否判定 end"}
    return status
  end


	# 在庫引当処理
  def calcu在庫引当処理()
    $Mlog.debug("B_DailyStockUpdate_model") {"calcu在庫引当処理 start"}                  # Logファイル Debug用
	
		sql_出荷入力     = getJsonChunkById(@sql_data, "sqls", "出荷数量集計", "input",  "record")		
		sql_出荷         = getJsonChunkById(@sql_data, "sqls", "出荷数量集計", "output", "record")		
		sql_単品在庫ＷＫ = getJsonChunkById(@sql_data, "sqls", "単品在庫計算用ＷＫ抽出", "output", "record")
		
		
		if sql_単品在庫ＷＫ["ref_単品ＩＤ"]["value"][0] == ""
		  if sql_出荷["当日出荷数量"]["value"][0] != ""
			  update在庫引当処理３(sql_出荷入力, sql_出荷)
			end
			
      $Mlog.debug("B_DailyPlanStockUpdate_model") {"calcu在庫引当処理 end"}                    # Logファイル Debug用
      return "OK"
		end
		
		
		if sql_出荷["当日出荷数量"]["value"][0] == ""
		  sql_出荷["当日出荷数量"]["value"][0] = "0"
		end
		
		
		max = sql_単品在庫ＷＫ["ref_単品ＩＤ"]["value"].length - 1
		for i in 0..max
		  # 最終行の処理
		  if i == max
			  update在庫引当処理２(sql_単品在庫ＷＫ, sql_出荷, i)
			  break
			end
			
			
			# 出荷数量と在庫数量の比較
			if sql_出荷["当日出荷数量"]["value"][0] > sql_単品在庫ＷＫ["当日在庫数量"]["value"][i]
			  update在庫引当処理１(sql_単品在庫ＷＫ, sql_出荷, i)
			else
			  update在庫引当処理２(sql_単品在庫ＷＫ, sql_出荷, i)
			  break
			end
		end
		
		
    $Mlog.debug("B_DailyStockUpdate_model") {"calcu在庫引当処理 end"}                    # Logファイル Debug用
    return "OK"
	end

	# 在庫引当処理１（当日出荷数量　＞　当日在庫数量　の時）
  def update在庫引当処理１(sql_単品在庫ＷＫ, sql_出荷, i)
    $Mlog.debug("B_DailyStockUpdate_model") {"update在庫引当処理１ start"}                  # Logファイル Debug用
	
		sql_出荷["当日出荷数量"]["value"][0]     = sql_出荷["当日出荷数量"]["value"][0].to_i - sql_単品在庫ＷＫ["当日在庫数量"]["value"][i].to_i

		sql_単品在庫ＷＫ["出荷数量"]["value"][i]     = sql_単品在庫ＷＫ["当日在庫数量"]["value"][i]
		sql_単品在庫ＷＫ["当日在庫数量"]["value"][i] = 0
		
		
		update単品在庫計算用ワーク更新(sql_単品在庫ＷＫ, i)
		
		
    $Mlog.debug("B_DailyStockUpdate_model") {"update在庫引当処理１ end"}                    # Logファイル Debug用
	end

	# 在庫引当処理２（当日出荷数量　＜＝　当日在庫数量　の時）
  def update在庫引当処理２(sql_単品在庫ＷＫ, sql_出荷, i)
    $Mlog.debug("B_DailyStockUpdate_model") {"update在庫引当処理２ start"}                  # Logファイル Debug用
	
		sql_単品在庫ＷＫ["出荷数量"]["value"][i]     = sql_出荷["当日出荷数量"]["value"][0]
		sql_単品在庫ＷＫ["当日在庫数量"]["value"][i] = sql_単品在庫ＷＫ["当日在庫数量"]["value"][i].to_i - sql_出荷["当日出荷数量"]["value"][0].to_i
		
		
		update単品在庫計算用ワーク更新(sql_単品在庫ＷＫ, i)
		
    $Mlog.debug("B_DailyStockUpdate_model") {"update在庫引当処理２ end"}                    # Logファイル Debug用
	end

	# 在庫引当処理３（出荷データが有り、在庫・入庫データが無い時）
  def update在庫引当処理３(sql_出荷入力, sql_出荷)
    $Mlog.debug("B_DailyStockUpdate_model") {"update在庫引当処理３ start"}                  # Logファイル Debug用
		
		w_品質維持可能日数 = sql_出荷["品質維持可能日数"]["value"][0]
		w_在庫日付         = sql_出荷入力["在庫日付"]["value"][0]
		w_品質維持期限１   = Date.strptime(w_在庫日付,'%Y/%m/%d') + w_品質維持可能日数
		w_品質維持期限２   = w_品質維持期限１.strftime('%Y/%m/%d')
    w_当日出荷数量     = 0 - sql_出荷["当日出荷数量"]["value"][0].to_i
	
    str_sql    = "INSERT INTO 単品在庫計算用ワーク ("
    str_sql   += "在庫日付, "
    str_sql   += "ref_単品ＩＤ, "
    str_sql   += "ref_発注ＩＤ, "
    str_sql   += "ref_行番号, "
    str_sql   += "前日在庫数量, "
    str_sql   += "入荷数量, "
    str_sql   += "出荷数量, "
    str_sql   += "廃棄数量, "
    str_sql   += "当日在庫数量, "
    str_sql   += "品質維持期限 "
    str_sql   += ") VALUES ("
    str_sql   += "'#{sql_出荷入力["在庫日付"]["value"][0]}', "
    str_sql   += " #{sql_出荷入力["単品ＩＤ"]["value"][0]}, "
    str_sql   += "0,"
    str_sql   += "999,"
    str_sql   += "0,"
    str_sql   += "0,"
    str_sql   += "#{sql_出荷["当日出荷数量"]["value"][0]},"
    str_sql   += "0,"
    str_sql   += "#{当日出荷数量},"
    str_sql   += "'#{w_品質維持期限２}'"
    str_sql   += ")"
		
		
    #$Mlog.debug("B_DailyStockUpdate_model") {"update単品予定在庫計算用ワーク１更新 doSQL : #{str_sql}"}     # Logファイル Debug用
    @rmenu_db["default"].doRun(str_sql)
		
    $Mlog.debug("B_DailyStockUpdate_model") {"defupdate在庫引当処理３ end"}                    # Logファイル Debug用
	end


	# 単品在庫計算用ワーク　更新
  def update単品在庫計算用ワーク更新(sql_単品在庫ＷＫ, i)
    $Mlog.debug("B_DailyStockUpdate_model") {"update単品在庫計算用ワーク更新 start"}                  # Logファイル Debug用
	
    str_sql    = "UPDATE 単品在庫計算用ワーク SET "
    str_sql   += "出荷数量     = #{sql_単品在庫ＷＫ["出荷数量"]["value"][i]}, "
    str_sql   += "当日在庫数量 = #{sql_単品在庫ＷＫ["当日在庫数量"]["value"][i]} "
    str_sql   += " WHERE "
    str_sql   += "在庫日付     = '#{sql_単品在庫ＷＫ["在庫日付"]["value"][0]}' AND "
    str_sql   += "ref_単品ＩＤ =  #{sql_単品在庫ＷＫ["ref_単品ＩＤ"]["value"][0]}  AND "
    str_sql   += "ref_入荷ＩＤ =  #{sql_単品在庫ＷＫ["ref_入荷ＩＤ"]["value"][0]}  AND "
    str_sql   += "ref_行番号   =  #{sql_単品在庫ＷＫ["ref_行番号"]["value"][0]} "
		
		
    $Mlog.debug("B_DailyStockUpdate_model") {"update単品在庫計算用ワーク更新 doSQL : #{str_sql}"}     # Logファイル Debug用
    @rmenu_db["default"].doRun(str_sql)
		
    $Mlog.debug("B_DailyStockUpdate_model") {"update単品在庫計算用ワーク更新 end"}                    # Logファイル Debug用
	end

end
