# coding: UTF-8

require 'RmenuConfig'                                                           # Rmenu環境設定クラス
require 'RmenuJsonMixin'                                                        # ID名で該当レコードを取得する
require 'RmenuLoggerMixin'                                                      # Logファイル Debug用
require 'base64'

class M_GoodDocsMainte_model
  include RmenuJsonMixin
  include RmenuLoggerMixin                                                      # Logファイル Debug用

  def initialize(rmenu_db, sql_data, request_data)
    begin
      # initialize 開始ログを出力する
      $Mlog.debug("M_GoodDocsMainte_model") {"initialize start"}                   # Logファイル Debug用

      @rmenu_db       = rmenu_db
      @sql_data       = sql_data
      @request_data   = request_data

      # initialize 終了ログを出力する
      $Mlog.debug("M_GoodDocsMainte_model") {"initialize normal end"}              # Logファイル Debug用
    rescue Exception
      # エラーログを出力する
      $Mlog.error("M_GoodDocsMainte_model") {"initialize exception: #{$!}"}        # Logファイル Debug用
      raise
    end
  end

  # 処理を実行するかの判定
  def checkDoSql(idName)
    $Mlog.debug("M_GoodDocsMainte_model") {"checkDoSql start"}                  # Logファイル Debug用

    detailRecord = getJsonChunkById(@request_data, "records", idName)

    status = "OK"
    if detailRecord["record"]["行番号"]["value"].length == 0
      status = "PASS"
    end

    $Mlog.debug("M_GoodDocsMainte_model") {"checkDoSql end"}                    # Logファイル Debug用
    return status
  end

  # ファイルタイプコードを設定する
  def setファイルタイプコードOfFlowerShop(filetypeID, dataID)
    $Mlog.debug("M_GoodDocsMainte_model") {"setファイルタイプコードOfFlowerShop start"}                  # Logファイル Debug用

		headerRecord   = getJsonChunkById(@request_data, "records", dataID)
    filetype2      = headerRecord["record"]["ファイルタイプ２"]["value"][0]

    filetypeRecord = getJsonChunkById(@sql_data, "sqls", filetypeID, "output")
    filetypecode   = filetypeRecord["record"]["ファイルタイプＩＤ"]["value"]
    filetype       = filetypeRecord["record"]["ファイルタイプ"]["value"]

    sqlRecord      = getJsonChunkById(@sql_data, "sqls", dataID, "input")
		
    filetype.each_with_index do |val, index|
		  valArray = val.split(":")
			if filetype2 == valArray[1]
			  sqlRecord["record"]["ファイルタイプ２"]["value"][0] = filetypecode[index]
				break
			end
		end

		$Mlog.debug("M_GoodDocsMainte_model") {"setファイルタイプコードOfFlowerShop end"}                    # Logファイル Debug用
    return "OK"
  end

  # アップロードしたファイルをＤＢに書き込む（新規）
  def writeDocsDataOf商品OfFlowerShop(idName)
    $Mlog.debug("M_GoodDocsMainte_model") {"writeDocsDataOf商品OfFlowerShop start"}                  # Logファイル Debug用

    headerRecord = getJsonChunkById(@request_data, "records", "header2")
    faleName     = headerRecord["record"]["ファイル名２"]["value"][0]
    url = $Rconfig["apps_path"] + "/FlowerShop/UpLoad/Apps/M_GoodDocsMainte/" + faleName

    # ファイルを読み込み、Data64でエンコードする
    imageData   = readBinFile(url)
    imageData64 = Base64.encode64(imageData)

    sqlRecord     = getJsonChunkById(@sql_data, "sqls", idName, "output")
    productFileID = sqlRecord["record"]["商品ドキュメントＩＤ"]["value"][0]

    # DBへ出力する
    str_sql   = "UPDATE 商品ドキュメント SET "
    str_sql  += "ドキュメントデータ = '#{imageData64}' WHERE 商品ドキュメントＩＤ = #{productFileID}"
    @rmenu_db["default"].update(str_sql)

    # ファイルを削除する
    deleteFile(url)

    $Mlog.debug("M_GoodDocsMainte_model") {"writeDocsDataOf商品OfFlowerShop end"}                    # Logファイル Debug用
    return "OK"
  end

  # アップロードしたファイルをＤＢに書き込む（変更）
  def updateDocsDataOf商品OfFlowerShop(idName)
    $Mlog.debug("M_GoodDocsMainte_model") {"updateDocsDataOf商品OfFlowerShop start"}                  # Logファイル Debug用

    headerRecord  = getJsonChunkById(@request_data, "records", idName)
    faleName      = headerRecord["record"]["ファイル名２"]["value"][0]
    productFileID = headerRecord["record"]["商品ドキュメントＩＤ２"]["value"][0]
    url           = $Rconfig["apps_path"] + "/FlowerShop/UpLoad/Apps/M_GoodDocsMainte/" + faleName

    # ファイルを読み込み、Data64でエンコードする
    imageData   = readBinFile(url)
    imageData64 = Base64.encode64(imageData)

    # DBへ出力する
    str_sql   = "UPDATE 商品ドキュメント SET "
    str_sql  += "ドキュメントデータ = '#{imageData64}' WHERE 商品ドキュメントＩＤ = #{productFileID}"
    @rmenu_db["default"].update(str_sql)

    # ファイルを削除する
    deleteFile(url)

    $Mlog.debug("M_GoodDocsMainte_model") {"updateDocsDataOf商品OfFlowerShop end"}                    # Logファイル Debug用
    return "OK"
  end

  # ＤＢから読み込みこんだイメージデータを、ファイルに出力する
  def downloadDocsDataOf商品OfFlowerShop(idName)
    $Mlog.debug("M_GoodDocsMainte_model") {"downloadDocsDataOf商品OfFlowerShop start"}                  # Logファイル Debug用

    sqlRecord = getJsonChunkById(@sql_data, "sqls", idName, "output")
    nameArray = sqlRecord["record"]["ファイル名２"]["value"][0].split(".")
		

    # 当日日付とカレント時刻を設定する
    w_date      = Date.today
	  w_time      = Time.now
		yyyymmdd    = w_date.strftime("%Y%m%d")
		hhmmss      = w_time.strftime("%H%M%S")
			
    # ディレクトリが無ければ作成する（有れば何もしない）
    downloadDir = $Rconfig['apps_path'] + "/FlowerShop/DownLoad/Apps/M_GoodDocsMainte"
    newDir      = $Rconfig['apps_path'] + "/FlowerShop/DownLoad/Apps/M_GoodDocsMainte" + "/d" + yyyymmdd
    createDir(newDir)
			
		# 当日よりも古いディレクトリーを削除する
		deleteDir(downloadDir, newDir)

		# イメージデータをBase64デコードし、ファイルへ出力する
    imageData64   = sqlRecord["record"]["ドキュメントデータ"]["value"][0]
    imageData     = Base64.decode64(imageData64)
    url = newDir + "/" + nameArray[0] + hhmmss + "." + nameArray[1]
    writeBinFile(url, imageData)

    # ダウンロード用パス＋ファイル名を設定する
		downloadfile = "FlowerShop/DownLoad/Apps/M_GoodDocsMainte" + "/d" + yyyymmdd + "/" + nameArray[0] + hhmmss + "." + nameArray[1]
    sqlRecord["record"]["ドキュメントデータ"]["value"][0] = downloadfile

    $Mlog.debug("M_GoodDocsMainte_model") {"downloadDocsDataOf商品OfFlowerShop end"}                    # Logファイル Debug用
    return "OK"
  end


end
