# coding: UTF-8

require 'RmenuJsonMixin'                                                        # ID名で該当レコードを取得する
require 'FlowerShop/Server/Modules/FlowerShopOfMasterMainteOfModelMixin'
require 'base64'
require 'fileutils'
require 'date'

class M_SearchItemDetail_model
  include RmenuJsonMixin
  include FlowerShopOfMasterMainteOfModelMixin

  def initialize(rmenu_dbi, sql_data, request_data)
    begin
      # initialize 開始ログを出力する
      $Mlog.debug("M_SearchItemDetail_model") {"initialize start"}                   # Logファイル Debug用

      @rmenu_dbi      = rmenu_dbi
      @sql_data       = sql_data
      @request_data   = request_data

      # initialize 終了ログを出力する
      $Mlog.debug("M_SearchItemDetail_model") {"initialize normal end"}              # Logファイル Debug用
    rescue Exception
      # エラーログを出力する
      $Mlog.error("M_SearchItemDetail_model") {"initialize exception: #{$!}"}        # Logファイル Debug用
      raise
    end
  end

  # SQL文の検索条件を変更する
  def setSql_単品検索OfFlowerShop(idname1, idname2 ,genesql_freesql)
    $Mlog.debug("M_SearchItemDetail") {"setSql_単品検索OfFlowerShop start"}                  # Logファイル Debug用

    requestInfo  = getJsonChunkById(@request_data, "records", idname1)

    searchString01  = requestInfo["record"]["検索単品名"]["value"][0]
    searchString02  = requestInfo["record"]["検索仕入先ＩＤ"]["value"][0]

    sqlInfo      = getJsonChunkById(@sql_data, "sqls", idname2)

    rep = ""

    # 検索単品名
    if searchString01 != ""
      rep = "単品名称 LIKE '%#{searchString01}%'"
    end

    # 検索仕入先ＩＤ
    if searchString02 != ""
		  if rep == ""
        rep = "ref_仕入先ＩＤ = #{searchString02}"
			else
        rep = rep + "AND ref_仕入先ＩＤ = #{searchString02}"
			end
    end

    # 検索条件の置き換え
    if    genesql_freesql == 'genesql'
      sqlInfo["sql"]["genesql"]["where"].sub!(/&&/,rep)
    elsif genesql_freesql == 'freesql'
      strJoin = sqlInfo["sql"]["freesql"].join("\t")
      strJoin.sub!(/&&/,rep)
      strSplit = strJoin.split("\t")
      sqlInfo["sql"]["freesql"].each_with_index do |temp, idx|
        sqlInfo["sql"]["freesql"][idx] = strSplit[idx]
      end
    end

    $Mlog.debug("M_SearchItemDetail") {"setSql_単品検索OfFlowerShop end"}                    # Logファイル Debug用
    return "OK"
  end

  # ＤＢから読み込みこんだイメージデータを、ファイルに出力する
  def downloadImageDataOf単品検索OfFlowerShop(idName)
    $Mlog.debug("M_SearchItemDetail") {"downloadImageDataOf単品検索OfFlowerShop start"}                  # Logファイル Debug用

    # 当日日付とカレント時刻を設定する
    w_date      = Date.today
	  w_time      = Time.now
		yyyymmdd    = w_date.strftime("%Y%m%d")
		hhmmss      = w_time.strftime("%H%M%S")
			
    # ディレクトリが無ければ作成する（有れば何もしない）
    downloadDir = $Rconfig['apps_path'] + "/FlowerShop/DownLoad/Apps/M_SearchItemDetail"
    newDir      = $Rconfig['apps_path'] + "/FlowerShop/DownLoad/Apps/M_SearchItemDetail" + "/d" + yyyymmdd
    createDir(newDir)
			
		# 当日よりも古いディレクトリーを削除する
		deleteDir(downloadDir, newDir)
		

    sqlRecord  = getJsonChunkById(@sql_data, "sqls", idName, "output")
    nameArray  = sqlRecord["record"]["画像ファイル名"]["value"]
    imageArray = sqlRecord["record"]["画像データ"]["value"]
		
    nameArray.each_with_index { |val, index| 
      if val != ""
        if imageArray[index] != ""
				  fileArray = nameArray[index].split(".")

		      # イメージデータをBase64デコードし、ファイルへ出力する
          imageData64   = imageArray[index]
          imageData     = Base64.decode64(imageData64)
          url = newDir + "/" + fileArray[0] + hhmmss + "." + fileArray[1]
          writeBinFile(url, imageData)

          # ダウンロード用パス＋ファイル名を設定する
		      downloadfile = "FlowerShop/DownLoad/Apps/M_SearchItemDetail" + "/d" + yyyymmdd + "/" + fileArray[0] + hhmmss + "." + fileArray[1]
          sqlRecord["record"]["画像データ"]["value"][index] = downloadfile
        else
          sqlRecord["record"]["画像データ"]["value"][index] = ""
        end
      else
        sqlRecord["record"]["画像データ"]["value"][index] = ""
      end
    }


    $Mlog.debug("M_SearchItemDetail") {"downloadImageDataOf単品検索OfFlowerShop end"}                    # Logファイル Debug用
    return "OK"
  end

end
