# coding: UTF-8

require 'date'
require 'RmenuFilePathMixin'                                                    # セッション処理用
require 'RmenuJsonMixin'                                                        # ID名で該当レコードを取得する

class M_SetItemDetail_view
  include RmenuFilePathMixin                                                    # Logファイル Debug用
  include RmenuJsonMixin

  def initialize(response_data, sql_data, request_data)
    begin
      # initialize 開始ログを出力する
      $Vlog.debug("M_SetItemDetail_view") {"initialize start"}                    # Logファイル Debug用

      @response_data  = response_data
      @sql_data       = sql_data
      @request_data   = request_data
      
      # initialize 終了ログを出力する
      $Vlog.debug("M_SetItemDetail_view") {"initialize normal end"}               # Logファイル Debug用
    rescue Exception
      # エラーログを出力する
      $Vlog.error("M_SetItemDetail_view") {"initialize exception: #{$!}"}         # Logファイル Debug用
      raise
    end
  end

  # 総件数とページ当りの行数を使って、総ページ数を計算する
  def getMaxPageCount
    $Vlog.debug("M_SetItemDetail_view") {"getMaxPageCount start"}                 # Logファイル Debug用

    response_record = @response_data["records"][0]["record"]
    page_line       = response_record["ページライン数"]["value"][0].to_i
    total_count     = response_record["トータル件数"]["value"][0].to_i
    if total_count == 0
      max_page      = 1
    else
      max_page      = ((total_count + (page_line - 1)) / page_line).truncate
    end
    
    response_record["最大ページ"]["value"][0] = max_page
    
    $Vlog.debug("M_SetItemDetail_view") {"getMaxPageCount end"}                   # Logファイル Debug用
    return "OK"
  end

  # CSVファイル作成
  def createCSVFile
    $Vlog.debug("M_SetItemDetail_model") {"createCSVFile start"}                  # Logファイル Debug用

    sqlInfo   = getJsonChunkById(@sql_data, "sqls", "csvCreate")
    sqlRecord = sqlInfo["output"]["record"]

    # 見出し出力
    data = ""
    max  = 0
    i    = 0
    sqlRecord.each do |name, value|
      if i == 0
        max = value["value"].length - 1
      else
        data << ","
      end
      data << name
      i += 1
    end
    data << "\n"
 
    # データ出力
    for i in 0..max
      j = 0
      sqlRecord.each do |name, value|
        if j != 0
          data << ","
        end
        data << value["value"][i]
        j += 1
      end
      data << "\n"
    end

    # ファイル出力
    tempTime   = Time.now
    tempNow    = tempTime.strftime("%Y%m%d%H%M%S")
    csvDir     = $Rconfig['apps_path'] + "/" + @response_data["html"].gsub(/\/Json\//, '/DownLoad/')
    csvNewFile = csvDir + "/" + "data" + "#{tempNow}.csv"

    writeCodeFile(csvNewFile, data, "Shift_JIS")

    # ファイル情報設定
    csvPath       = @response_data["html"].gsub(/\/Json\//, '/DownLoad/')
    filename = "data" + "#{tempNow}.csv"

    responseInfo  = getJsonChunkById(@response_data, "records", "csvCreate")
    responseInfo["record"]["downloadfile"]["value"][0] = csvPath + "/" + filename

    $Vlog.debug("M_SetItemDetail_model") {"createCSVFile end"}                    # Logファイル Debug用
    return "OK"
  end

end
