# coding: UTF-8

require 'RmenuJsonMixin'                                                        # ID名で該当レコードを取得する
require 'RmenuLoggerMixin'                                                      # Logファイル Debug用
require 'base64'
require 'fileutils'
require 'date'


class T_ReceiveOrderUpdate_model
  include RmenuJsonMixin
  include RmenuLoggerMixin                                                      # Logファイル Debug用

  def initialize(rmenu_dbi, sql_data, request_data)
    begin
      # initialize 開始ログを出力する
      $Mlog.debug("T_ReceiveOrderUpdate_model") {"initialize start"}                   # Logファイル Debug用

      @rmenu_dbi      = rmenu_dbi
      @sql_data       = sql_data
      @request_data   = request_data

      # initialize 終了ログを出力する
      $Mlog.debug("T_ReceiveOrderUpdate_model") {"initialize normal end"}              # Logファイル Debug用
    rescue Exception
      # エラーログを出力する
      $Mlog.error("T_ReceiveOrderUpdate_model") {"initialize exception: #{$!}"}        # Logファイル Debug用
      raise
    end
  end

  # Base64Encode
  def setBase64EncodeOf受注訂正OfFlowerShop(reqIdName, reqItemName1, reqItemName2, reqItemName3, reqItemName4, sqlIdName, sqlItemName)
    $Mlog.debug("T_ReceiveOrderUpdate_model") {"setBase64EncodeOf受注訂正OfFlowerShop start"}                  # Logファイル Debug用

    requestInfo = getJsonChunkById(@request_data, "records", reqIdName)
    sqlInfo     = getJsonChunkById(@sql_data, "sqls", sqlIdName, "input")
		
		w_クレジット番号１ = requestInfo["record"]["#{reqItemName1}"]["value"][0]
		w_クレジット番号２ = requestInfo["record"]["#{reqItemName2}"]["value"][0]
		w_クレジット番号３ = requestInfo["record"]["#{reqItemName3}"]["value"][0]
		w_クレジット番号４ = requestInfo["record"]["#{reqItemName4}"]["value"][0]
		w_クレジット番号   = w_クレジット番号１ + "-" + w_クレジット番号２ + "-" + w_クレジット番号３ + "-" + w_クレジット番号４
		
		w_クレジット番号９ = Base64.encode64(w_クレジット番号)
		
		
    sqlInfo["record"]["#{sqlItemName}"]["value"][0] = w_クレジット番号９
		
    $Mlog.debug("T_ReceiveOrderUpdate_model") {"setBase64EncodeOf受注訂正OfFlowerShop end"}                    # Logファイル Debug用
    return "OK"
  end

  # Base64Decode
  def setBase64DecodeOf受注訂正OfFlowerShop(sqlIdName, sqlItemName)
    $Mlog.debug("T_ReceiveOrderUpdate_model") {"setBase64DecodeOf受注訂正OfFlowerShop start"}                  # Logファイル Debug用

    sqlInfo     = getJsonChunkById(@sql_data, "sqls", sqlIdName, "output")
		
		w_クレジット番号 = Base64.decode64(sqlInfo["record"]["#{sqlItemName}"]["value"][0])
		
    sqlInfo["record"]["#{sqlItemName}"]["value"][0] = w_クレジット番号
		
    $Mlog.debug("T_ReceiveOrderUpdate_model") {"setBase64DecodeOf受注訂正OfFlowerShop end"}                    # Logファイル Debug用
    return "OK"
  end

  # バッチ処理キーの作成
  def createバッチ処理キーOf受注入力訂正()
    $Mlog.debug("T_ReceiveOrderUpdate_model") {"createバッチ処理キーOf受注入力訂正 start"}

    status = "OK"

    sqlRecord1  = getJsonChunkById(@sql_data, "sqls", "バッチキー作成１",    "output", "record")
    sqlRecord2  = getJsonChunkById(@sql_data, "sqls", "バッチキー作成２",    "output", "record")
    sqlRecord3  = getJsonChunkById(@sql_data, "sqls", "parallelKey",         "input",  "record")
		
		max単品ＩＤ = sqlRecord1["単品ＩＤ"]["value"].length - 1
		
    key単品ＩＤ = [""]
    key引当日   = [""]
		

    if sqlRecord1["単品ＩＤ"]["value"][0] == ""
		  sqlRecord3["単品ＩＤ"]["value"] = key単品ＩＤ
		  sqlRecord3["引当日"]["value"]   = key引当日
			
      $Mlog.debug("T_ReceiveOrderUpdate_model") {"createバッチ処理キーOf受注入力訂正 end"}
      return status
		end
		
		
		j = 0
		for i in 0..max単品ＩＤ do
		  if sqlRecord1["単品ＩＤ"]["value"][i] == ""
			  break
			end
			
			
			if sqlRecord1["最大出荷予定日付"]["value"][i] > sqlRecord2["最大在庫日付"]["value"][0]
			  max最大日 = sqlRecord1["最大出荷予定日付"]["value"][i]
			else
			  max最大日 = sqlRecord2["最大在庫日付"]["value"][0]
			end
			
			
			if sqlRecord1["最小出荷予定日付"]["value"][i] < sqlRecord2["最大在庫日付"]["value"][0]
		    dateワーク１ = Date.strptime(sqlRecord1["最小出荷予定日付"]["value"][i],'%Y/%m/%d')
			else
		    dateワーク１ = Date.strptime(sqlRecord2["最大在庫日付"]["value"][0],'%Y/%m/%d')
			end
			

		  loop do
			
				dateワーク２ = dateワーク１.strftime("%Y/%m/%d")
				if dateワーク２ > max最大日
				  break
				end
				
        key単品ＩＤ[j] = sqlRecord1["単品ＩＤ"]["value"][i]
        key引当日[j]   = dateワーク２
				j = j + 1
			  dateワーク１ = dateワーク１ + 1
				
		  end
			
		end
		
		sqlRecord3["単品ＩＤ"]["value"] = key単品ＩＤ
		sqlRecord3["引当日"]["value"]   = key引当日
		
    $Mlog.debug("T_ReceiveOrderUpdate_model") {"createバッチ処理キーOf受注入力訂正 end"}
    return status
  end


  # バッチ実行可否判定
  def checkバッチ実行可否判定(sqlIdName, sqlIO, sqlItemName)
    $Mlog.debug("T_ReceiveOrderUpdate_model") {"checkバッチ実行可否判定 start"}

    status = "OK"

    cond_chunk = getJsonChunkById(@sql_data, "sqls", sqlIdName, sqlIO, "record")
    checkValue = cond_chunk[sqlItemName]["value"][0]
		
    if checkValue == ""
      status = "ALLPASS"
    end

    $Mlog.debug("T_ReceiveOrderUpdate_model") {"checkバッチ実行可否判定: #{sqlIdName} #{sqlIO} #{sqlItemName} #{checkValue} #{status}"}
    $Mlog.debug("T_ReceiveOrderUpdate_model") {"checkバッチ実行可否判定 end"}
    return status
  end


end
