# coding: UTF-8

require 'date'
require 'RmenuLoggerMixin'                                                      # Logファイル Debug用
require 'RmenuJsonMixin'                                                        # ID名で該当レコードを取得する

class BatchDailyProcessing_model
  include RmenuLoggerMixin                                                      # Logファイル Debug用
  include RmenuJsonMixin

  def initialize(rmenu_dbi, sql_data, request_data)
    begin
      # initialize 開始ログを出力する
      $Mlog.debug("BatchDailyProcessing") {"initialize start"}                   # Logファイル Debug用

      @rmenu_dbi      = rmenu_dbi
      @sql_data       = sql_data
      @request_data   = request_data
      
      # initialize 終了ログを出力する
      $Mlog.debug("BatchDailyProcessing") {"initialize normal end"}              # Logファイル Debug用
    rescue Exception
      # エラーログを出力する
      $Mlog.error("BatchDailyProcessing") {"initialize exception: #{$!}"}        # Logファイル Debug用
      raise
    end
  end
	

  # バッチ売上伝票番号管理を設定する
  def setバッチ売上伝票番号管理OfRmenuSales(idName, itemName)
    $Mlog.debug("SalesInput_model") {"setバッチ売上伝票番号管理OfRmenuSales start"}                  # Logファイル Debug用

    requestInfo  = getJsonChunkById(@request_data, "records", idName)
		w_会計年度   = requestInfo["record"]["会計年度"]["value"][0]
    w_年         = w_会計年度[2, 2].to_i * 10000
		
    # データが無い時、登録する
    str_sql1   = "SELECT 伝票番号 FROM 伝票番号管理 WHERE 年度 = '#{w_会計年度}' AND 伝票種類 = '売上'"
    w_伝票番号 = @rmenu_db["default"].select_value(str_sql1)

    $Mlog.debug("SalesInput_model") {"setバッチ売上伝票番号管理OfRmenuSales 伝票番号　＝　#{w_伝票番号}"}                  # Logファイル Debug用

    if !w_伝票番号
      str_sql2 = "INSERT INTO 伝票番号管理 (年度, 伝票種類, 伝票番号, 更新者, 更新日時, 削除フラグ)"
      str_sql3 = " VALUES ('#{w_会計年度}', '売上', #{w_年}, 'システム', CURRENT_TIMESTAMP, '0')"
      @rmenu_db["default"].insert(str_sql2 + str_sql3)
    end

    $Mlog.debug("SalesInput_model") {"setバッチ売上伝票番号管理OfRmenuSales end"}                    # Logファイル Debug用
    return "OK"
  end

  # バッチ売上年月日を設定する
  def setバッチ売上年月日(idName1, itemName1, idName2, itemName2)
    $Mlog.debug("SalesInput_model") {"setバッチ売上年月日 start"}                  # Logファイル Debug用

    sqlInfo1  = getJsonChunkById(@sql_data, "records", idName1)
    sqlInfo2  = getJsonChunkById(@sql_data, "records", idName2)
		
		w_dayObject    = Date.today
		w_システム日付 = w_dayObject.strftime("%Y/%m/%d")
		w_契約年月日   = sqlInfo1["output"]["record"][#{itemName1}]["value"][0]
		
		if w_契約年月日 == w_システム日付
		  sqlInfo2["input"]["record"][#{itemName2}]["value"][0] = w_システム日付
      return "OK"
		end
		

		# 日付を配列に変換
		w_システムArray = w_システム日付.split("/")
		w_契約Array     = w_契約年月日.split("/")
		
		
		# 契約年月日の末日を取得する
		w_契約末日1 = Date.new(w_契約Array[0], w_契約Array[1], -1)
		w_契約末日2 = w_契約末日1.strftime("%Y/%m/%d")
		
		# 契約年月日が末日のとき
		if w_契約年月日 == w_契約末日2
		  w_当月末日1 = Date.new(w_システムArray[0], w_システムArray[1], -1)
		  w_当月末日2 = w_当月末日1.strftime("%Y/%m/%d")
		  sqlInfo2["input"]["record"][#{itemName2}]["value"][0] = w_当月末日2
      return "OK"
		end
		
		# 末日以外の時
		w_売上年月日 = w_システムArray[0] + "/" + w_システムArray[1] + "/" + w_契約Array[2]
		sqlInfo2["input"]["record"][#{itemName2}]["value"][0] = w_売上年月日
		
    $Mlog.debug("SalesInput_model") {"setバッチ売上年月日 end"}                    # Logファイル Debug用
    return "OK"
  end

	
end
