# coding: UTF-8

require 'fileutils'
require 'RmenuLoggerMixin'                                                      # Logファイル Debug用
require 'RmenuFilePathMixin'                                                    # Logファイル Debug用
require 'RmenuJsonMixin'                                                        # ID名で該当レコードを取得する

class LogsViewer_model
  include RmenuLoggerMixin                                                      # Logファイル Debug用
  include RmenuFilePathMixin                                                    # Logファイル Debug用
  include RmenuJsonMixin

  def initialize(rmenu_dbi, sql_data, request_data)
    begin
      # initialize 開始ログを出力する
      $Mlog.debug("LogsViewer_model") {"initialize start"}                   # Logファイル Debug用

      @rmenu_dbi      = rmenu_dbi
      @sql_data       = sql_data
      @request_data   = request_data

      # initialize 終了ログを出力する
      $Mlog.debug("LogsViewer_model") {"initialize normal end"}              # Logファイル Debug用
    rescue Exception
      # エラーログを出力する
      $Mlog.error("LogsViewer_model") {"initialize exception: #{$!}"}        # Logファイル Debug用
      raise
    end
  end


  # ラックログを取得する
  def getラックログ()
    $Mlog.debug("LogsViewer_model") {"getラックログ start"}                  # Logファイル Debug用

    sqlRecord       = getJsonChunkById(@sql_data,     "sqls",    "resource", "output", "record")
		
		# ラックログ　読込む
		racklogPath     = $Rconfig['logs_path'] + "/rack.log"
    racklogdata     = getラックログ情報(racklogPath)

    # 開始時刻・終了時刻　読込む
		datetimeArray = get開始時刻(racklogPath)
		
		sqlRecord["ラックログ"]["value"]   = racklogdata
		sqlRecord["開始時刻"]["value"]     = datetimeArray
		sqlRecord["終了時刻"]["value"]     = datetimeArray
				
    $Mlog.debug("LogsViewer_model") {"getラックログ end"}                    # Logファイル Debug用
    return "OK"
  end

	
  # ラックログを取得する
  def getログ情報()
    $Mlog.debug("LogsViewer_model") {"getログ情報 start"}                  # Logファイル Debug用

    requestRecord  = getJsonChunkById(@request_data, "records", "searchCondition",    "record")
    sqlRecord      = getJsonChunkById(@sql_data,     "sqls",    "resource", "output", "record")
		
		w_StartTime    = requestRecord["開始時刻１"]["value"][0]
		w_EndTime      = requestRecord["終了時刻１"]["value"][0]
		
		# コントローラログ　読込む
		logfilePath    = $Rconfig['logs_path'] + "/controller.log"
		sqlRecord["コントローラログ"]["value"] = getその他ログ情報(logfilePath, w_StartTime, w_EndTime)
		
		# モデルログ　読込む
		logfilePath     = $Rconfig['logs_path'] + "/model.log"
		sqlRecord["モデルログ"]["value"] = getその他ログ情報(logfilePath, w_StartTime, w_EndTime)
		
		# モデルＤＢログ　読込む
		logfilePath     = $Rconfig['logs_path'] + "/modelDB.log"
		sqlRecord["モデルＤＢログ"]["value"] = getその他ログ情報(logfilePath, w_StartTime, w_EndTime)
		
		# ビューログ　読込む
		logfilePath     = $Rconfig['logs_path'] + "/view.log"
		sqlRecord["ビューログ"]["value"] = getその他ログ情報(logfilePath, w_StartTime, w_EndTime)

    $Mlog.debug("LogsViewer_model") {"getログ情報 end"}                    # Logファイル Debug用
    return "OK"
  end

	
	def get開始時刻(filePath)
    $Mlog.debug("LogsViewer_model") {"get開始時刻 start"}                  # Logファイル Debug用

    begin
		  datetimeArray   = Array.new
			w_datetime      = ""
			
      File.open(filePath) do |file|
        file.each_line do |data|
				  idx = data.index("[")
					
			    if idx
			      newStr = data[idx + 1, 19]
				
		    		if w_datetime == newStr
				      next
				    end
			
			      datetimeArray.push(newStr)
			    	w_datetime = newStr
          end
        end
      end
    rescue SystemCallError => e
      $Mlog.debug("LogsViewer_model") {"get開始時刻 エラー ： class=[#{e.class}] message=[#{e.message}]"}
    rescue IOError => e
      $Mlog.debug("LogsViewer_model") {"get開始時刻 エラー ： class=[#{e.class}] message=[#{e.message}]"}
    end
		
    $Mlog.debug("LogsViewer_model") {"get開始時刻 end"}
		return datetimeArray
	end
	
	
	def getラックログ情報(logfilePath)
    $Mlog.debug("LogsViewer_model") {"getラックログ情報 start"}                  # Logファイル Debug用

    begin
		  logData = Array.new
			
      File.open(logfilePath, 'r:utf-8') do |file|
        file.each_line do |data|
					logData.push(data)
        end
      end
    rescue SystemCallError => e
      $Mlog.debug("LogsViewer_model") {"getラックログ情報 エラー ： class=[#{e.class}] message=[#{e.message}]"}
    rescue IOError => e
      $Mlog.debug("LogsViewer_model") {"getラックログ情報 エラー ： class=[#{e.class}] message=[#{e.message}]"}
    end
		
    $Mlog.debug("LogsViewer_model") {"getラックログ情報 end"}
		return logData
	end
	
	def getその他ログ情報(logfilePath, w_StartTime, w_EndTime)
    $Mlog.debug("LogsViewer_model") {"getその他ログ情報 start"}                  # Logファイル Debug用

    begin
		  logData = Array.new
			outSW   = "0"
			
      File.open(logfilePath, 'r:utf-8') do |file|
        file.each_line do |data|
				  dFlag    = data[3, 1]
					if dFlag != "["
					  if outSW == "1"
						  logData.push(data[0, 99999])
						end
						
						next
					end
						
					
				  timeData = data[4, 19]
					
					if timeData < w_StartTime
					　outSW   = "0"
					  next
					end
					
					if timeData > w_EndTime
					  outSW   = "0"
					  next
					end
					
					logData.push(data[47, 99999])
					outSW = "1"
        end
      end
    rescue SystemCallError => e
      $Mlog.debug("LogsViewer_model") {"getその他ログ情報 エラー ： class=[#{e.class}] message=[#{e.message}]"}
    rescue IOError => e
      $Mlog.debug("LogsViewer_model") {"getその他ログ情報 エラー ： class=[#{e.class}] message=[#{e.message}]"}
    end
		
    $Mlog.debug("LogsViewer_model") {"getその他ログ情報 end"}
		return logData
	end
	
end
