# coding: UTF-8

require 'fileutils'
require 'RmenuLoggerMixin'                                                      # Logファイル Debug用
require 'RmenuJsonMixin'                                                        # ID名で該当レコードを取得する

class ViewerJson_model
  include RmenuLoggerMixin                                                      # Logファイル Debug用
  include RmenuJsonMixin

  def initialize(rmenu_dbi, sql_data, request_data)
    begin
      # initialize 開始ログを出力する
      $Mlog.debug("ViewerJson_model") {"initialize start"}                   # Logファイル Debug用

      @rmenu_dbi      = rmenu_dbi
      @sql_data       = sql_data
      @request_data   = request_data

      # initialize 終了ログを出力する
      $Mlog.debug("ViewerJson_model") {"initialize normal end"}              # Logファイル Debug用
    rescue Exception
      # エラーログを出力する
      $Mlog.error("ViewerJson_model") {"initialize exception: #{$!}"}        # Logファイル Debug用
      raise
    end
  end
	
  # プロジェクト名を取得する
  def getプロジェクト(idname)
    $Mlog.debug("ViewerJson_model") {"getプロジェクト start"}                  # Logファイル Debug用

    sqlRecord       = getJsonChunkById(@sql_data, "sqls", idname, "output", "record")
		
		dirArray = Dir::entries($Rconfig['apps_path'])
		i        = 0
		dirArray.each{|dirName|
		  if dirName == "."
			  next
			end
		  if dirName == ".."
			  next
			end
			
			pos = dirName.index(".")
			if pos
			  next
			end
			
			sqlRecord["選択プロジェクト"]["value"][i]   = dirName
			sqlRecord["選択プロジェクト名"]["value"][i] = dirName
			i = i + 1
    }
		
				
    $Mlog.debug("ViewerJson_model") {"getプロジェクト end"}                    # Logファイル Debug用
    return "OK"
  end
	

  # プログラム名を取得する
  def getプログラム(reqName, sqlName)
    $Mlog.debug("ViewerJson_model") {"getプログラム start"}                  # Logファイル Debug用

    requestRecord   = getJsonChunkById(@request_data, "records", reqName,           "record")
    sqlRecord       = getJsonChunkById(@sql_data,     "sqls",    sqlName, "output", "record")
		
		projectName     = requestRecord["プロジェクト名"]["value"][0]
		
		dirArray = Dir::entries($Rconfig['apps_path'] + "/" + projectName + "/Html/Apps")
		i        = 0
		dirArray.each{|dirName|
		  if dirName == "."
			  next
			end
		  if dirName == ".."
			  next
			end
			
			pos = dirName.index(".")
			if pos
			  next
			end
			
			sqlRecord["選択プログラム"]["value"][i]   = dirName
			sqlRecord["選択プログラム名"]["value"][i] = dirName
			i = i + 1
    }
		
				
    $Mlog.debug("ViewerJson_model") {"getプログラム end"}                    # Logファイル Debug用
    return "OK"
  end

	
  # モード（トランザクション）名を取得する
  def getモード(reqName, sqlName)
    $Mlog.debug("ViewerJson_model") {"getモード start"}                  # Logファイル Debug用

    requestRecord   = getJsonChunkById(@request_data, "records", reqName,           "record")
    sqlRecord       = getJsonChunkById(@sql_data,     "sqls",    sqlName, "output", "record")
		
		projectName     = requestRecord["プロジェクト名"]["value"][0]
		programName     = requestRecord["プログラム名"]["value"][0]
		
		jsonArray = Dir::entries($Rconfig['apps_path'] + "/" + projectName + "/Json/Apps/" + programName)
		i        = 0
		jsonArray.each{|jsonName|
		  if jsonName == "."
			  next
			end
		  if jsonName == ".."
			  next
			end
			
			pos = jsonName.index(".")
			if !pos
			  next
			end
			
			modeArray = jsonName.split("_")
			maxSize   = modeArray.length - 1
			if maxSize < 2
			  next
			end
			
			if modeArray[maxSize] == "sql.json"
			  next
			end			
			
			
			sqlRecord["選択モード"]["value"][i]   = modeArray[maxSize - 1]
			sqlRecord["選択モード名"]["value"][i] = modeArray[maxSize - 1]
			i = i + 1
    }
		
				
    $Mlog.debug("ViewerJson_model") {"getモード end"}                    # Logファイル Debug用
    return "OK"
  end


  # ＨＴＭＬソースを取得する
  def getＨＴＭＬソース(reqName, sqlName)
    $Mlog.debug("ViewerJson_model") {"getＨＴＭＬソース start"}                  # Logファイル Debug用

    requestRecord   = getJsonChunkById(@request_data, "records", reqName,           "record")
    sqlRecord       = getJsonChunkById(@sql_data,     "sqls",    sqlName, "output", "record")
		
		projectName     = requestRecord["プロジェクト名"]["value"][0]
		programName     = requestRecord["プログラム名"]["value"][0]
		
		htmlPath        = $Rconfig['apps_path'] + "/" + projectName + "/Html/Apps/" + programName + "/index.html"
		htmlSource      = readHtml(htmlPath)
		
		sqlRecord["ＨＴＭＬソース"]["value"][0]   = htmlSource
				
    $Mlog.debug("ViewerJson_model") {"getＨＴＭＬソース end"}                    # Logファイル Debug用
    return "OK"
  end
	

  # ＪＳソースを取得する
  def getＪＳソース(reqName, sqlName)
    $Mlog.debug("ViewerJson_model") {"getＪＳソース start"}                  # Logファイル Debug用

    requestRecord   = getJsonChunkById(@request_data, "records", reqName,           "record")
    sqlRecord       = getJsonChunkById(@sql_data,     "sqls",    sqlName, "output", "record")
		
		projectName     = requestRecord["プロジェクト名"]["value"][0]
		programName     = requestRecord["プログラム名"]["value"][0]
		
		jsonArray = Dir::entries($Rconfig['apps_path'] + "/" + projectName + "/Html/Apps/" + programName)
		jsonHash = {}
		jsonArray.each{|jsonName|
		  if jsonName == "."
			  next
			end
		  if jsonName == ".."
			  next
			end
			
			pos = jsonName.index(".")
			if !pos
			  next
			end

			jsArray = jsonName.split(".")
			maxSize = jsArray.length - 1
			if jsArray[maxSize] != "js"
			  next
			end
			
			
		  jsonPath             = $Rconfig['apps_path'] + "/" + projectName + "/Html/Apps/" + programName + "/" + jsonName
		  jsonSource           = readHtml(jsonPath)
			jsonHash[jsonName]   = jsonSource

		}
		
		sqlRecord["ＪＳソース"]["value"][0] = jsonHash.to_json
		
				
    $Mlog.debug("ViewerJson_model") {"getＪＳソース end"}                    # Logファイル Debug用
    return "OK"
  end
	

  # ＪＳＯＮソースを取得する
  def getＪＳＯＮソース(reqName, sqlName)
    $Mlog.debug("ViewerJson_model") {"getＪＳＯＮソース start"}                  # Logファイル Debug用

    requestRecord   = getJsonChunkById(@request_data, "records", reqName,           "record")
    sqlRecord       = getJsonChunkById(@sql_data,     "sqls",    sqlName, "output", "record")
		
		projectName     = requestRecord["プロジェクト名"]["value"][0]
		programName     = requestRecord["プログラム名"]["value"][0]
		
		jsonArray = Dir::entries($Rconfig['apps_path'] + "/" + projectName + "/Json/Apps/" + programName)
		jsonHash = {}
		jsonArray.each{|jsonName|
		  if jsonName == "."
			  next
			end
		  if jsonName == ".."
			  next
			end
			
			pos = jsonName.index(".")
			if !pos
			  next
			end
			
		  jsonPath             = $Rconfig['apps_path'] + "/" + projectName + "/Json/Apps/" + programName + "/" + jsonName
		  jsonSource           = readHtml(jsonPath)
			jsonHash[jsonName]   = jsonSource

		}
		
		sqlRecord["ＪＳＯＮソース"]["value"][0] = jsonHash.to_json
		
    $Mlog.debug("ViewerJson_model") {"getＪＳＯＮソース end"}                    # Logファイル Debug用
    return "OK"
  end
	

  # Ｒｕｂｙソースを取得する
  def getＲｕｂｙソース(reqName, sqlName)
    $Mlog.debug("ViewerJson_model") {"getＲｕｂｙソース start"}                  # Logファイル Debug用

    requestRecord   = getJsonChunkById(@request_data, "records", reqName,           "record")
    sqlRecord       = getJsonChunkById(@sql_data,     "sqls",    sqlName, "output", "record")
		
		projectName     = requestRecord["プロジェクト名"]["value"][0]
		programName     = requestRecord["プログラム名"]["value"][0]
		
		jsonArray = Dir::entries($Rconfig['apps_path'] + "/" + projectName + "/Server/Apps/" + programName)
		jsonHash = {}
		i        = 0
		jsonArray.each{|jsonName|
		  if jsonName == "."
			  next
			end
		  if jsonName == ".."
			  next
			end
			
			pos = jsonName.index(".")
			if !pos
			  next
			end
			
		  jsonPath             = $Rconfig['apps_path'] + "/" + projectName + "/Server/Apps/" + programName + "/" + jsonName
		  jsonSource           = readHtml(jsonPath)
			jsonHash[jsonName]   = jsonSource
			
		  sqlRecord["選択ルビー"]["value"][i]   = jsonName
		  sqlRecord["選択ルビー名"]["value"][i] = jsonName
      i = i + 1
			
		}
		
		sqlRecord["Ｒｕｂｙソース"]["value"][0] = jsonHash.to_json
		
				
    $Mlog.debug("ViewerJson_model") {"getＲｕｂｙソース end"}                    # Logファイル Debug用
    return "OK"
  end
	

  # ＤＢ名を取得する
  def getＤＢ名(reqName, sqlName)
    $Mlog.debug("ViewerJson_model") {"getＤＢ名 start"}                  # Logファイル Debug用

    requestRecord   = getJsonChunkById(@request_data, "records", reqName,           "record")
    sqlRecord       = getJsonChunkById(@sql_data,     "sqls",    sqlName, "output", "record")
		
		projectName     = requestRecord["プロジェクト名"]["value"][0]
		programName     = requestRecord["プログラム名"]["value"][0]
		
		jsonArray = Dir::entries($Rconfig['apps_path'] + "/" + projectName + "/Json/Apps/" + programName)
		jsonHash = {}
		jsonArray.each{|jsonName|
		  if jsonName == "."
			  next
			end
		  if jsonName == ".."
			  next
			end
			
			pos = jsonName.index(".")
			if !pos
			  next
			end

			sqlArray = jsonName.split("_")
			maxSize = sqlArray.length - 1
			if sqlArray[maxSize] != "sql.json"
			  next
			end
			
			
		  jsonPath    = $Rconfig['apps_path'] + "/" + projectName + "/Json/Apps/" + programName + "/" + jsonName
		  jsonSource  = readHtml(jsonPath)
			sqlObject   = JSON.parse(jsonSource)
			
			jsonHash["dbname"] = sqlObject["dbname"]
			break

		}
		
		sqlRecord["ＤＢ名"]["value"][0] = jsonHash["dbname"]
		
		
				
    $Mlog.debug("ViewerJson_model") {"getＤＢ名 end"}                    # Logファイル Debug用
    return "OK"
  end
	

  # ＤＢ名を設定する
  def setＤＢ名(reqName, sqlName)
    $Mlog.debug("ViewerJson_model") {"setＤＢ名 start"}                  # Logファイル Debug用

    requestRecord = getJsonChunkById(@request_data, "records", reqName,           "record")
    sqlInfo       = getJsonChunkById(@sql_data,     "sqls",    sqlName)
		
		databaseName      = requestRecord["検索ＤＢ名"]["value"][0]
		sqlInfo["dbname"] = databaseName	
		
				
    $Mlog.debug("ViewerJson_model") {"setＤＢ名 end"}                    # Logファイル Debug用
    return "OK"
  end
	


end
