# coding: UTF-8

require 'rexml/parsers/streamparser'
require 'rexml/parsers/baseparser'
require 'rexml/streamlistener'
require 'RmenuVisualTools/Server/Modules/RmenuVisualToolsOfSVGListener'
require 'RmenuLoggerMixin'                                                      # Logファイル Debug用
require 'base64'
require 'fileutils'
require 'date'

module RmenuVisualToolsOfCreateSVGFileOfModelMixin
  include RmenuLoggerMixin                                                      # Logファイル Debug用

  # ＳＶＧデータのViewBoxを再計算し、新しい値を設定する
  def getSVGFile(svgData, svgFileName, htmlPath)
    begin
      # 画像データを解析し、viewBoxを算出する
      svglistener = RmenuVisualToolsOfSVGListener.new
      REXML::Parsers::StreamParser.new(svgData, svglistener).parse
      viewBox     = svglistener.getViewBox()

      # viewBoxの開始位置と終了位置を取得し、再計算したviewBoxに置き換える
      pos1 = svgData.index('viewBox="')
      pos2 = svgData.index('"', pos1 + 10)
      svgData[pos1..pos2] = viewBox

      # 当日日付とカレント時刻を設定する
      w_date      = Date.today
			w_time      = Time.now
			yyyymmdd    = w_date.strftime("%Y%m%d")
			hhmmss      = w_time.strftime("%H%M%S")
			
      # ディレクトリが無ければ作成する（有れば何もしない）
      downloadDir = $Rconfig['apps_path'] + "/" + htmlPath.gsub(/\/Json\//, '/DownLoad/')
      newDir      = $Rconfig['apps_path'] + "/" + htmlPath.gsub(/\/Json\//, '/DownLoad/') + "/d" + yyyymmdd
      createDir(newDir)
			
			# 当日よりも古いディレクトリーを削除する
			deleteDir(downloadDir, newDir)

      # SVGファイル出力
      svgNewFile = newDir + "/" + svgFileName + hhmmss + ".svg"
      writeBinFile(svgNewFile, svgData)

      # レスポンスファイル名を設定
      svgResponseFile = "d" + yyyymmdd + "/" + svgFileName + hhmmss + ".svg"

      return svgResponseFile
    rescue Exception
      return ""
    end
  end

  # SVGファイルを作成する（受注入力専用）
  def createSVGFile(idName)
    $Mlog.debug("RmenuVisualToolsOfCreateSVGFileOfModelMixin") {"createSVGFile start"}                  # Logファイル Debug用

    sqlInfo = getJsonChunkById(@sql_data, "sqls", idName)

    # 画像データを解析しviewBoxを算出、算出したviewBoxを書き換えたSVGファイルを出力する
    svgData     = sqlInfo["output"]["record"]["画像データ"]["value"]

    svgFileName = sqlInfo["input"]["record"]["管理番号"]["value"]
    htmlPath    = @sql_data["html"]
    svgFileName.each_with_index { |val, index| 
      if val
        if svgData[index]
          svgNewFile  = getSVGFile(Base64.decode64(svgData[index]), val, htmlPath)
          sqlInfo["output"]["record"]["downloadfile"]["value"][index] = svgNewFile
					svgData[index] = ""
        else
          sqlInfo["output"]["record"]["downloadfile"]["value"][index] = ""
        end
      else
        sqlInfo["output"]["record"]["downloadfile"]["value"][index] = ""
      end
    }

    $Mlog.debug("RmenuVisualToolsOfCreateSVGFileOfModelMixin") {"createSVGFile end"}                    # Logファイル Debug用
    return "OK"
  end

  # PNGデータを設定する
  def getPNGFile(svgData, svgFileName, htmlPath)
    begin

      # 当日日付とカレント時刻を設定する
      w_date      = Date.today
			w_time      = Time.now
			yyyymmdd    = w_date.strftime("%Y%m%d")
			hhmmss      = w_time.strftime("%H%M%S")
			
      # ディレクトリが無ければ作成する（有れば何もしない）
      downloadDir = $Rconfig['apps_path'] + "/" + htmlPath.gsub(/\/Json\//, '/DownLoad/')
      newDir      = $Rconfig['apps_path'] + "/" + htmlPath.gsub(/\/Json\//, '/DownLoad/') + "/d" + yyyymmdd
      createDir(newDir)
			
			# 当日よりも古いディレクトリーを削除する
			deleteDir(downloadDir, newDir)

      # SVGファイル出力
      svgNewFile = newDir + "/" + svgFileName + hhmmss + ".png"
      writeBinFile(svgNewFile, svgData)

      # レスポンスファイル名を設定
      svgResponseFile = "d" + yyyymmdd + "/" + svgFileName + hhmmss + ".png"

      return svgResponseFile
    rescue Exception
      return ""
    end
  end

  # PNGファイルを作成する（受注入力専用）
  def createPNGFile(idName)
    $Mlog.debug("RmenuVisualToolsOfCreateSVGFileOfModelMixin") {"createPNGFile start"}                  # Logファイル Debug用

    sqlInfo = getJsonChunkById(@sql_data, "sqls", idName)

    # 画像データを解析しviewBoxを算出、算出したviewBoxを書き換えたSVGファイルを出力する
    svgData     = sqlInfo["output"]["record"]["画像データ"]["value"]

    svgFileName = sqlInfo["input"]["record"]["管理番号"]["value"]
    htmlPath    = @sql_data["html"]
    svgFileName.each_with_index { |val, index| 
      if val
        if svgData[index]
          svgNewFile  = getPNGFile(Base64.decode64(svgData[index]), val, htmlPath)
          sqlInfo["output"]["record"]["downloadfile"]["value"][index] = svgNewFile
        else
          sqlInfo["output"]["record"]["downloadfile"]["value"][index] = ""
        end
      else
        sqlInfo["output"]["record"]["downloadfile"]["value"][index] = ""
      end
    }

		sqlInfo["output"]["record"]["画像データ"]["value"] = ""
		
    $Mlog.debug("RmenuVisualToolsOfCreateSVGFileOfModelMixin") {"createPNGFile end"}                    # Logファイル Debug用
    return "OK"
  end

  # イメージデータをダウンロードファイルに出力する
  def getImageFile(imageData, imageFileName, mimeType, htmlPath)
    begin
      # 当日日付とカレント時刻を設定する
      w_date      = Date.today
			w_time      = Time.now
			yyyymmdd    = w_date.strftime("%Y%m%d")
			hhmmss      = w_time.strftime("%H%M%S")
			
      # ディレクトリが無ければ作成する（有れば何もしない）
      downloadDir = $Rconfig['apps_path'] + "/" + htmlPath.gsub(/\/Json\//, '/DownLoad/')
      newDir      = $Rconfig['apps_path'] + "/" + htmlPath.gsub(/\/Json\//, '/DownLoad/') + "/d" + yyyymmdd
      createDir(newDir)
			
			# 当日よりも古いディレクトリーを削除する
			deleteDir(downloadDir, newDir)

      # イメージデータファイル出力
      imageNewFile = newDir + "/" + imageFileName + hhmmss + "." + mimeType
      writeBinFile(imageNewFile, imageData)

      # レスポンスファイル名を設定
      imageResponseFile = "d" + yyyymmdd + "/" + imageFileName + hhmmss + "." + mimeType

      return imageResponseFile
    rescue Exception
      return ""
    end
  end

end
