# coding: UTF-8

require 'rexml/parsers/streamparser'
require 'rexml/parsers/baseparser'
require 'rexml/streamlistener'

# viewBox 再計算クラス
class RmenuVisualToolsOfSVGListener
    include REXML::StreamListener

    def tag_start(tag, attrs)
      case tag
      when "svg"              # 処理開始
        @minX    = 0
        @minY    = 0
        @maxX    = 0
        @maxY    = 0
        @width   = 0
        @height  = 0
      when "rect"
        rect(attrs)
      when "polyline"
        polyline(attrs)
      when "polygon"
        polygon(attrs)
      when "line"
        line(attrs)
      else
        return
      end
    end

    def tag_end tag
      case tag
      when "svg"              # 処理終了
        @width  = @maxX - @minX
        @height = @maxY - @minY
      else
        return
      end
    end

    # rectタグ
    def rect(attrs)
      if attrs["x"] && attrs["y"] && attrs["width"] && attrs["height"]
        x1 = attrs["x"].to_i
        y1 = attrs["y"].to_i
        x2 = x1 + attrs["width"].to_i
        y2 = y1 + attrs["height"].to_i

        if x1 <= 0 || y1 <= 0
          return
        end
        if x2 <= 0 || y2 <= 0
          return
        end

        if @minX == 0 && @minY == 0
          @minX = x1
          @minY = y1
        end

        if x1 < @minX
          @minX = x1
        end
        if y1 < @minY
          @minY = y1
        end
        if x2 > @maxX
          @maxX = x2
        end
        if y2 > @maxY
          @maxY = y2
        end
      end
    end

    # polylineタグ
    def polyline(attrs)
      array1 = attrs["points"].split(" ")
      array1.each{|value|
        array2 = value.split(",")
        x      = array2[0].to_i
        y      = array2[1].to_i

        if x <= 0 || y <= 0
          return
        end

        if @minX == 0 && @minY == 0
          @minX = x
          @minY = y
        end

        if x < @minX
          @minX = x
        end
        if y < @minY
          @minY = y
        end
        if x > @maxX
          @maxX = x
        end
        if y > @maxY
          @maxY = y
        end
      }
    end

    # polygonタグ
    def polygon(attrs)
      array1 = attrs["points"].split(" ")
      array1.each{|value|
        array2 = value.split(",")
        x      = array2[0].to_i
        y      = array2[1].to_i

        if x <= 0 || y <= 0
          return
        end

        if @minX == 0 && @minY == 0
          @minX = x
          @minY = y
        end

        if x < @minX
          @minX = x
        end
        if y < @minY
          @minY = y
        end
        if x > @maxX
          @maxX = x
        end
        if y > @maxY
          @maxY = y
        end
      }
    end

    # lineタグ
    def line(attrs)
      if attrs["x1"] && attrs["y1"] && attrs["x2"] && attrs["y2"]
        x1 = attrs["x1"].to_i
        y1 = attrs["y1"].to_i
        x2 = attrs["x2"].to_i
        y2 = attrs["y2"].to_i

        if x1 <= 0 || y1 <= 0
          return
        end
        if x2 <= 0 || y2 <= 0
          return
        end

        if x1 > x2
          w1 = x1
          w2 = x2
          x1 = w2
          x2 = w1
        end
        if y1 > y2
          w1 = y1
          w2 = y2
          y1 = w2
          y2 = w1
        end

        if x1 == 0 || y1 == 0
          return
        end
        if x2 == 0 || y2 == 0
          return
        end

        if @minX == 0 && @minY == 0
          @minX = x1
          @minY = y1
        end

        if x1 < @minX
          @minX = x1
        end
        if y1 < @minY
          @minY = y1
        end
        if x2 > @maxX
          @maxX = x2
        end
        if y2 > @maxY
          @maxY = y2
        end
      end
    end

    # 再計算したViewBoxを設定
    def getViewBox()
      viewBox = "viewBox=\"#{@minX.to_s} #{@minY.to_s} #{@width.to_s} #{@height.to_s}\""
      return viewBox
    end
end
