# coding: UTF-8

require 'RmenuLoggerMixin'
require 'RmenuParallelDBMixin'
require 'RmenuJsonMixin'

require 'json'
require 'RmenuTupleSpace'
require 'RmenuMainDatabase'
require 'RmenuParallelControll'
require 'RmenuParallelStartUp'

class RmenuTupleParallelMainServer
  include RmenuLoggerMixin
  include RmenuParallelDBMixin
  include RmenuJsonMixin

  def initialize(log_file_name)
    $Tlog               = createLogger(log_file_name)                           # controller用ログの作成

    begin
      # initialize 開始ログを出力する
      $Tlog.debug("TupleParallelMainServer") {"initialize start"}               # Logファイル Debug用
    
      @tuplespace    = RmenuTupleSpace.new()                                    # タプルスペース
      maindatabase   = RmenuMainDatabase.new(log_file_name + "DB")              # DATABASEオブジェクトをプーリング
      @db_queue      = maindatabase.getDatabase()

    
      # initialize 終了ログを出力する
      $Tlog.debug("TupleParallelMainServer") {"initialize end"}                 # Logファイル Debug用
    rescue
      # エラーログを出力する
      $Tlog.debug("TupleParallelMainServer") {"initialize exception: #{$!}"}    # Logファイル Debug用
    end
  end

  # メイン処理（タプル並列クライアントから呼び出される）
  def call(request, controll_id, sql_data)
    begin
      # メイン処理 開始ログを出力する
      $Tlog.debug("TupleParallelMainServer") {"call start"}                      # Logファイル Debug用

			if request == "tupleclient_start"
        # タプルスペース take(['TupleClient', parallel_controll_id, parallel_division_id, dbname])
        tuple = @tuplespace.takeTupleClient()
				
				# 並列クライアント　開始処理
				tupleclient_start(tuple)

        # メイン処理 終了ログを出力する
        $Tlog.debug("TupleParallelMainServer") {"call normal end"}                 # Logファイル Debug用
	      return tuple
	    end
			
			if request == "tupleclient_end"
        # 並列分散管理テーブルの分割処理終了回数と終了日時を更新し、NEXT処理フラッグを戻す
        nextStatus = tupleclient_end(controll_id, sql_data)

        # メイン処理 終了ログを出力する
        $Tlog.debug("TupleParallelMainServer") {"call normal end"}                 # Logファイル Debug用
				return nextStatus
			end
			
		  if request == "tupleserver"
			  # 次のジョブを起動する
			  status = nextjob_start(sql_data)

        # メイン処理 終了ログを出力する
        $Tlog.debug("TupleParallelMainServer") {"call normal end"}                 # Logファイル Debug用
			  return status
			end
			
    rescue
      # メイン処理 エラーログを出力する
      $Tlog.debug("TupleParallelMainServer") {"call exception: #{$@}"}           # Logファイル Debug用
    end
  end
	
	# 並列クライアント　開始処理
	def tupleclient_start(tuple)
    begin
        parallel_controll_id  = tuple[1]
        parallel_division_id  = tuple[2]
        dbname                = tuple[3]

        if parallel_division_id != 1
				  return "OK"
        end

				# DBオブジェクトをキューから取得する
        rmenu_db              = @db_queue.pop
        rmenuParallelControll = RmenuParallelControll.new(rmenu_db)               # ジョブ管理テーブル

        # トランザクションをスタートする
        $Tlog.debug("TupleMainClient") {"トランザクション スタート"}
        transaction(rmenu_db, dbname)
	
        $Tlog.debug("TupleMainClient") {"並列分散管理テーブルの開始日時を更新 start"}
        rmenuParallelControll.update_start_date(parallel_controll_id)
        $Tlog.debug("TupleMainClient") {"並列分散管理テーブルの開始日時を更新 end"}

  			# トランザクションをコミットする
        $Tlog.debug("TupleParallelMainServer") {"コミットを実行"}
        commit(rmenu_db)
					
				# DBオブジェクトをキューに戻す
        @db_queue.push(rmenu_db)
				
				return "OK"
    rescue
      # メイン処理 エラーログを出力する
      $Tlog.debug("TupleParallelMainServer") {"call exception: #{$@}"}           # Logファイル Debug用
    end
	end
	
	# 並列クライアント　終了処理
	def tupleclient_end(controll_id, sql_data)
    begin
        # DBオブジェクトをキューから取得し、DB名をsql_dataから取得する
        rmenu_db              = @db_queue.pop
			  dbname                = sql_data["dbname"]
        rmenuParallelControll = RmenuParallelControll.new(rmenu_db)               # ジョブ管理テーブル

				# トランザクションをスタートする
        $Tlog.debug("TupleParallelMainServer") {"トランザクション スタート"}
        transaction(rmenu_db, dbname)
				
        $Tlog.debug("TupleMainClient") {"並列分散管理テーブルの分割処理終了回数と終了日時を更新 start"}
        nextStatus = rmenuParallelControll.update_end_number(controll_id)
        $Tlog.debug("TupleMainClient") {"並列分散管理テーブルの分割処理終了回数と終了日時を更新 end"}
				
        # トランザクションをコミットする
        $Tlog.debug("TupleParallelMainServer") {"コミットを実行"}
        commit(rmenu_db)
					
				# DBオブジェクトをキューに戻す
        @db_queue.push(rmenu_db)
				
	      return nextStatus
    rescue
      # メイン処理 エラーログを出力する
      $Tlog.debug("TupleParallelMainServer") {"call exception: #{$@}"}           # Logファイル Debug用
    end
	end
	
	# 次のジョブを起動する
	def nextjob_start(sql_data)
    begin
        # DBオブジェクトをキューから取得し、DB名をsql_dataから取得する
        rmenu_db         = @db_queue.pop
		    parallelControll = getJsonChunkById(sql_data, "sqls", "parallelControll")
			  dbname           = parallelControll["dbname"]

				# トランザクションをスタートする
        $Tlog.debug("TupleParallelMainServer") {"トランザクション スタート"}
        transaction(rmenu_db, dbname)

				$Tlog.debug("TupleParallelMainServer") {"NEXT処理の起動 start"}
        parallelStartUp = RmenuParallelStartUp.new(rmenu_db)
        parallelStartUp.call(sql_data)
        $Tlog.debug("TupleParallelMainServer") {"NEXT処理の起動 end"}

        # トランザクションをコミットする
        $Tlog.debug("TupleParallelMainServer") {"コミットを実行"}
        commit(rmenu_db)
					
				# DBオブジェクトをキューに戻す
        @db_queue.push(rmenu_db)
				
	      return "OK"
    rescue
      # メイン処理 エラーログを出力する
      $Tlog.debug("TupleParallelMainServer") {"call exception: #{$@}"}           # Logファイル Debug用
    end
	end
	
end
